;;;--- Original program to demonstrate the usage for the getCellsFunction
;;;    By JefferyPSanders.com
;;;--- Function to retrieve values for a cell or a range of cell
;;;
;;;    To retrieve a cells value:
;;;   (openexcel "D:\\1Waseemtarik\\Hardware 3.xlxs" "Sheet1")
;;;   (setq filename "D:\\1Waseemtarik\\Book1.xlsx" sheetname "Sheet1")
;;;

;   Count (RO) = 1

; Count will be 0 if no excel open but if no workbooks also may return same value. Nil names. 
; So a double check count /=0 and wb not ""

; Get cell
; (vlax-get-property  (vlax-get Ws 'Cells) 'Item 1  5 )

; ColumnRow - Returns a list of the Column and Row number
; Function By: Gilles Chanteau from Marseille, France
; Arguments: 1
;   Cell$ = Cell ID
; Syntax example: (ColumnRow "ABC987") = '(731 987)
;default to "A1" if there's a problem
;-------------------------------------------------------------------------------
(defun ColumnRow (Cell$ / Column$ Char$ Row#)
  (setq Column$ "")
  (while (< 64 (ascii (setq Char$ (strcase (substr Cell$ 1 1)))) 91)
    (setq Column$ (strcat Column$ Char$)
          Cell$ (substr Cell$ 2)
    )
  )
  (if (and (/= Column$ "") (numberp (setq Row# (read Cell$))))
    (list (Alpha2Number Column$) Row#)
    '(1 1)
  )
)

; Alpha2Number - Converts Alpha string into Number
; Function By: Gilles Chanteau from Marseille, France
; Arguments: 1
;   Str$ = String to convert
; Syntax example: (Alpha2Number "ABC") = 731
;-------------------------------------------------------------------------------
(defun Alpha2Number (Str$ / Num#)
  (if (= 0 (setq Num# (strlen Str$)))
    0
    (+ (* (- (ascii (strcase (substr Str$ 1 1))) 64) (expt 26 (1- Num#)))
       (Alpha2Number (substr Str$ 2))
    )
  )
)
;-------------------------------------------------------------------------------
; Number2Alpha - Converts Number into Alpha string
; Function By: Gilles Chanteau from Marseille, France
; Arguments: 1
;   Num# = Number to convert
; Syntax example: (Number2Alpha 731) = "ABC"
;-------------------------------------------------------------------------------
(defun Number2Alpha (Num# / Val#)
  (if (< Num# 27)
    (chr (+ 64 Num#))
    (if (= 0 (setq Val# (rem Num# 26)))
      (strcat (Number2Alpha (1- (/ Num# 26))) "Z")
      (strcat (Number2Alpha (/ Num# 26)) (chr (+ 64 Val#)))
    )
  )
);defun Number2Alpha

; rtosr - Used to change a real number into a short real number string
; stripping off all trailing 0's.
; Arguments: 1
;   RealNum~ = Real number to convert to a short string real number
; Returns: ShortReal$ the short string real number value of the real number.
;-------------------------------------------------------------------------------
(defun rtosr (RealNum~ / DimZin# ShortReal$)
  (setq DimZin# (getvar "DIMZIN"))
  (setvar "DIMZIN" 8)
  (setq ShortReal$ (rtos RealNum~ 2 8))
  (setvar "DIMZIN" DimZin#)
  ShortReal$
);defun rtosr

; thanks to Lee-mac for this defun
; 58 is Colon

(defun csv->lst ( str del / pos )
(if (setq pos (vl-string-position del str))
    (cons (substr str 1 pos) (csv->lst (substr str (+ pos 2))))
    (list str)
    )
)

; thanks to Lee-mac for this defun 
; www.lee-mac.com
; 44 is comma 9 is tab 34 is space 58 is colon
(defun _csv->lst58 ( str / pos )
	(if (setq pos (vl-string-position 58 str))
		(cons (substr str 1 pos) (_csv->lst58 (substr str (+ pos 2))))
		(list str)
    )
)


	 

(defun openexist ( / )
(setq aH:ex "T")
(setq mySheet (vl-catch-all-apply 'vlax-get-property (list (vlax-get-property myxl "Sheets") "Item" sheetName)))
(vlax-invoke-method mySheet "Activate")
)


(defun openexcel-sheetname (filename sheetname / )
(setq myBook (vl-catch-all-apply 'vla-open (list (vlax-get-property myXL "WorkBooks") fileName)))
(setq mySheet (vl-catch-all-apply 'vlax-get-property (list (vlax-get-property myBook "Sheets") "Item" sheetName)))
(vlax-invoke-method mySheet "Activate")
)

(defun newexcelname (newname / )
(vlax-invoke-method (vlax-get-property myXL 'WorkBooks) 'Add) ; opens a new xl
(vlax-put-property  (vlax-get-property (VLAX-get-property myxl "ActiveWorkbook") 'name) newname)
)

; set current worksheet  needs work
(defun setsheet ( sheetname / )
(setq xlSheets (vlax-get-property myxl "Worksheets"))
(setq curSheet (vlax-get-property xlSheets "Item" 1))  ; 1 is 1st sheet need names of sheets somehow.
; (vlax-get cursheet 'Name) returns name 
(vlax-invoke-method curSheet "Activate")
)


; get hyperlinks
(defun hyp ( / )
(or (setq myxl (vlax-get-object "Excel.Application"))
    (setq myxl (vlax-get-or-create-object "excel.Application"))
)
(vla-put-visible myXL :vlax-true)
(vlax-put-property myxl 'ScreenUpdating :vlax-true)
(vlax-put-property myXL 'DisplayAlerts :vlax-true)
(setq xlSheets (vlax-get-property myxl "Worksheets"))
(setq curSheet (vlax-get-property xlSheets "Item" 1)); 1 is 1st sheet need names of sheets somehow.
; (vlax-get cursheet 'Name) ; returns name
(setq hyp (VLAX-GET CURSHEET 'HYPERLINKS))
(setq hyp1 (vlax-get-property hyp "Item" 1))
(setq linkname (vlax-get hyp1 'Name))
(setq desc (vlax-get hyp1 'texToDisplay))
(alert (strcat linkname "\n\n" desc))
(princ)
)

(or (setq wks (vlax-get myxl 'ActiveSheet))
    (vlax-invoke (vlax-get myxl 'workbooks) 'Add)
)
(vlax-put (vlax-get-property wks 'range "B1") 'value "Length")

(defun putcell (cellname val1 / )
(setq myRange (vlax-get-property  (vlax-get-property myxl "ActiveSheet") "Range" cellname))
(vlax-put-property myRange 'Value2 val1)
)

;;	Thanks to fixo			;;
;;   = Set Excel cell text =    ;;
;;				;;
(defun xlsetcelltext ( row column text)
(setq cells (vlax-get-property  (vlax-get-property myxl "ActiveSheet") "Cells"))
  (vl-catch-all-apply
    'vlax-put-property
    (list cells 'Item row column
	(vlax-make-variant (vl-princ-to-string text) vlax-vbstring)))
)



(defun getcell (cellname / )
(setq myRange (vlax-get-property  (vlax-get-property myxl "ActiveSheet") "Range" cellname))
(princ  (vlax-variant-value (vlax-get-property myRange 'Value2)))
)


;;	Thanks to fixo			;;
(defun getcell2 (row column / )
(setq cells (vlax-get-property  (vlax-get-property myxl "ActiveSheet") "Cells"))
(setq cell (vlax-get (vlax-variant-value  (vlax-get-property cells "Item" row column)) 'value))
)


(defun putimagecell (cellname val1 / )
(setq myRange (vlax-get-property  (vlax-get-property myxl "ActiveSheet") "Range" cellname))
(vlax-put-property myRange 'Picture val1)
)

; get active range selected
(defun getrangexl ( / lst UR CR RADD )

(setq lst '())
(setq UR (vlax-get-property  (vlax-get-property myxl "ActiveSheet") "UsedRange"))
(setq CR (vlax-get-property UR "CurrentRegion"))
(setq RADD (vlax-get-property CR "Address"))
(setq cnt (vlax-get-property CR  "Count"))

(setq lst (_csv->lst58 radd))

(setq st (vl-string-subst "" "$" (vl-string-subst "" "$" (nth 0 lst) )))
(setq end (vl-string-subst "" "$" (vl-string-subst "" "$" (nth 1 lst) )))

(setq st  (columnrow st))
(setq end  (columnrow end))

(princ st)
(princ "\n")
(princ end)

)

; this code was done by the great FIXO
; no longer with us

; if needed
;(setq ExcelValue
;(cond
;((= (type ExcelValue) 'INT) (itoa ExcelValue))
;((= (type ExcelValue) 'REAL) (rtosr ExcelValue))
;((= (type ExcelValue) 'STR) (vl-string-trim " " ExcelValue))
;((/= (type ExcelValue) 'STR) "")
;)
;)


(defun getrangexl2 ( / lst UR CR RADD )
(vl-catch-all-error-p
	   (setq Rng
		  (vl-catch-all-apply
		    (function (lambda ()
				(vlax-variant-value
				  (vlax-invoke-method
				    (vlax-get-property myxl 'Application)
				    'Inputbox
				    "Select a Range: "
				    "Range Selection "
				    nil
				    nil
				    nil
				    nil
				    nil
				    8))))))
)
(setq xrng (vlax-get-property rng "address"))
(setq xxrng xrng)
(repeat 4 (setq xxrng(vl-string-subst "" "$" xxrng)))
(setq xxxrng (_csv->lst58 xxrng))
(setq rngst (columnrow (nth 0 xxxrng)) rngend (columnrow (nth 1 xxxrng)))
(setq *ExcelData@ nil )
(setq Row# (nth 1 rngst))
(repeat (+ (- (nth 1 rngend)(nth 1 rngst) ) 1)
(setq Data@ nil)
(setq Column# (nth 0 rngst))
(repeat (+  (- (nth  0 rngend)(nth 0 rngst) ) 1)
(setq Range$ (strcat (Number2Alpha Column#)(itoa Row#)))
(setq ExcelRange (vlax-get-property myxl "Range" range$))
(setq ExcelVariant (vlax-get-property ExcelRange 'Value))
(setq ExcelValue (vlax-variant-value ExcelVariant))
(setq Data@ (append Data@ (list ExcelValue)))
(setq Column# (1+ Column#))
)
(setq *ExcelData@ (append *ExcelData@ (list Data@)))
(setq Row# (1+ Row#))
)
(princ)
)

; get range of entered cells ver 3
; then use _csv->lst58 to split.
(setq myrange (vlax-get-property (vlax-get-property (vlax-get-property  myxl "ActiveSheet") 'UsedRange) 'address))
(setq range (_csv->lst58 myrange))

(setq rangest (car range))
(repeat 4
(setq rangest (vl-string-subst "" "$" rangest))
)
(setq rangest (ColumnRow rangest))

(setq rangeend (cadr range))
(repeat 4
(setq rangeend (vl-string-subst "" "$" rangeend))
)
(setq rangeend (ColumnRow rangeend))

; getrangexl2

(defun getallcells ( /  )
(setq CurRegion (vlax-get-property (vlax-get-property
(vlax-get-property  myxl "ActiveSheet") "Range" "A1") "CurrentRegion")
)
(setq MaxRow# (vlax-get-property (vlax-get-property CurRegion "Rows") "Count"))
(setq MaxColumn# (vlax-get-property (vlax-get-property CurRegion "Columns") "Count"))
   (setq *ExcelData@ nil)
  (setq Row# 1)
  (repeat MaxRow#
    (setq Data@ nil)
    (setq Column# 1)
    (repeat MaxColumn#
      (setq Range$ (strcat (Number2Alpha Column#)(itoa Row#)))
   (setq ExcelRange^ (vlax-get-property *ExcelApp% "Range" Range$))
   (setq ExcelVariant^ (vlax-get-property ExcelRange^ 'Value))
   (setq ExcelValue (vlax-variant-value ExcelVariant^))
   (setq ExcelValue
     (cond
    ((= (type ExcelValue) 'INT) (itoa ExcelValue))
    ((= (type ExcelValue) 'REAL) (rtosr ExcelValue))
    ((= (type ExcelValue) 'STR) (vl-string-trim " " ExcelValue))
    ((/= (type ExcelValue) 'STR) "")
     )
   )
   (setq Data@ (append Data@ (list ExcelValue)))
   (setq Column# (1+ Column#))
    )
    (setq *ExcelData@ (append *ExcelData@ (list Data@)))
    (setq Row# (1+ Row#))
  )
(princ)
)



; add a sheetName by FIXO no longer with us
(defun addsheet ()
(setq  xlSheets (vlax-get-property myBook "Sheets"))
;; add new sheet
(vlax-invoke-method xlsheets 'Add nil nil nil)
;; get the number of sheets
(setq xlCount (vlax-get-property xlSheets 'Count))
;; get the newly created sheet
(setq xlSheet (vlax-get-property xlSheets "Item" (vlax-get-property (vlax-get-property myXL 'ActiveSheet) 'Name)))
;; change name of sheet to what you need
(vlax-put-property xlSheet 'Name "NewSheetName")
;; move sheet to the end of book
(vlax-invoke-method xlSheet 'Move
(vlax-get-property xlSheets "Item"
(vlax-get-property xlSheets 'Count)))
)

(defun closeexcel ( / )
(if (= ah:ex "T")
(vlax-invoke-method (vlax-get-property myxl "ActiveWorkbook") 'Close :vlax-False)
(vlax-invoke-method (vlax-get-property myxl "ActiveWorkbook")  "SaveAs" filename -4143 "" "" :vlax-false :vlax-false nil )
)

; (vlax-invoke-method myxl 'Quit)

(if (not (vlax-object-released-p myRange))(progn(vlax-release-object myRange)(setq myRange nil)))
(if (not (vlax-object-released-p mySheet))(progn(vlax-release-object mySheet)(setq mySheet nil)))
(if (not (vlax-object-released-p myBook))(progn(vlax-release-object myBook)(setq myBook nil)))
(if (not (vlax-object-released-p myXL))(progn(vlax-release-object myXL)(setq myXL nil)))
)


;   Count (RO) = 1

; Count will be 0 if no excel open but if no workbooks also may return same value. Nil names. 
; So a double check count /=0 and wb not ""


(defun chkexcel (filename  /  )
(or (setq myxl (vlax-get-object "Excel.Application"))
    (setq myxl (vlax-get-or-create-object "excel.Application"))
)
; (vlax-invoke-method (vlax-get-property myXL 'WorkBooks) 'Add)
(vla-put-visible myXL :vlax-true)
(vlax-put-property myxl 'ScreenUpdating :vlax-true)
(vlax-put-property myXL 'DisplayAlerts :vlax-true)

(if (/=  (setq wbook (vlax-get myxl "workbooks")) nil)
(progn
(setq cnt (vla-get-count wbook))
(if (> cnt 0)
(vlax-for wb wbook
	(If (=    (vla-get-fullname wb) filename)
	(progn (setq xl wb) (openexist))
	(setq ah:ex2 "N")
	)
)
(openexcel)
)
(if (= ah:ex2 "N")
(openexist)
)
)
)
(princ)
)

; (setq fname (getfiled "Select a Excel File" "c:/" "xl*" 8))
; (vla-put-visible myXL :vlax-true)
; (vlax-put-property myxl 'ScreenUpdating :vlax-true)
; (vlax-put-property myXL 'DisplayAlerts :vlax-true)
(startapp "Excel" fname)

(defun isopenexcel (fname  / ans lst)
(or (setq myxl (vlax-get-object "Excel.Application"))
    (setq myxl (vlax-get-or-create-object "excel.Application"))
)
(setq myBook (vl-catch-all-apply 'vla-open (list (vlax-get-property myXL "WorkBooks") fName)))
(setq xlSheets (vlax-get-property myBook "Sheets"))
(setq xlCount (vlax-get-property xlSheets 'Count))
(setq xlst '())
(vlax-for Worksheet xlsheets
   (setq xlst (cons (vlax-get-property Worksheet "Name") xlst))
)
(setq xlst (reverse xlst))
(setq xlst (cons "Please choose sheet" xlst))
(if (not AH:Butts)(load "Multi Radio buttons.lsp"))
(ah:butts 1 "V"  xlst)
(setq x 1)
(vlax-for Worksheet xlsheets
(if (= but x)
        (vlax-invoke-method Worksheet "Activate")
)
(setq x (1+ x))
)
(princ)
)


;; Save and close the Excel workbook
  (vl-catch-all-apply 'vlax-invoke-method myxl 'Save)

  ;; Close the Excel workbook without saving changes
  (vl-catch-all-apply 'vlax-invoke-method myxl 'Close :vlax-false)

  ;; Quit Excel application
  (vl-catch-all-apply 'vlax-invoke-method myxl 'Quit)


; Code by MunsteanStefan May 2023
; (vlax-get-property cells "Item" 5 2) = B5 row col

(setq ExFile "D:\\acadtemp\\book1.xlsx")
(if ExFile
    (progn
  (setq myxl  (vlax-get-or-create-object "Excel.Application"))
  (vla-put-visible myxl :vlax-true)
  (vlax-put-property myxl "ScreenUpdating" :vlax-true )
  (setq wbook-coll (vlax-get-property myxl "Workbooks"))
  (setq wbook  (vlax-invoke-method wbook-coll 'Open ExFile))
  (if (or (null wbook) (vl-catch-all-error-p wbook))
   (progn
   (alert "Can not find ExcelFile ")
   (vlax-put-property myxl "ScreenUpdating" :vlax-true )
   (vlax-invoke-method wbook-coll "close"  )
   (vlax-invoke-method myxl "quit"  )
   )
   (progn
  (setq sheet-coll (vlax-get-property wbook "Sheets"))
  (setq TruesheetDist (vlax-get-property sheet-coll "item" 1))
  (setq Cells (vlax-get TruesheetDist "cells"))
  (setq cell (vlax-variant-value (vlax-get-property cells "Item" 5 2)))
  (setq left (vlax-get cell "Left")
        top  (vlax-get cell "Top")
  )
  (setq shapes (vlax-get-property TruesheetDist "Shapes"))
  (vlax-invoke shapes
    "AddPicture"
    "C:\\VFT-CAD-TOOLS\\Images\\AlanH.png"
    :vlax-false
    :vlax-true
    left
    top
    -1
    -1
    )
  )
    )
  )
)

(defun FindValueInColumn (column value)
(setq mysheet (vl-catch-all-apply 'vlax-get-property (list (vlax-get-property mybook "Sheets") "Item" mytab)))
  ;; Get the range of cells in the specified column
  (setq range (vlax-get-property (vlax-get-property mysheet 'Columns) "Item" column))
  ;; Search for the value in the column
  (setq cell (vlax-invoke-method range 'Find value))
  ;; If the value is found, return the row number; otherwise, return nil
  (if cell
      (vlax-get-property cell 'Row)
    nil
  )
)